@@ -2,23 +2,19 @@
 
 
 from bisect import insort_right
-from typing import List, Any, Union, Optional, TypeVar, Dict, Tuple
+from typing import Any, Dict, List, Optional, Tuple, TypeVar, Union
 
-from regex import compile as regex_compile, VERBOSE
+from regex import VERBOSE, compile as regex_compile
 
 from ._cell import (
-    Cell,
-    NEWLINE_CELL_MATCH,
-    INLINE_HAEDER_CELL_MATCH,
-    INLINE_NONHAEDER_CELL_MATCH
-)
-from ._tag import SubWikiTextWithAttrs
+    Cell, INLINE_HAEDER_CELL_MATCH, INLINE_NONHAEDER_CELL_MATCH,
+    NEWLINE_CELL_MATCH)
 from ._spans import ATTRS_MATCH
+from ._tag import SubWikiTextWithAttrs
 from ._wikitext import WS
 
-
 CAPTION_MATCH = regex_compile(
-    r"""
+    rb"""
     # Everything until the caption line
     (?P<preattrs>
         # Start of table
@@ -39,14 +35,8 @@ CAPTION_MATCH = regex_compile(
         (?!\|)
     )?
     (?P<caption>.*?)
-    # End of caption line
-    (?:
-        \n|
-        \|\|
-    )
-    """,
-    VERBOSE
-).match
+    (?:\n|\|\|)
+    """, VERBOSE).match
 T = TypeVar('T')
 
 
@@ -185,8 +175,7 @@ class Table(SubWikiTextWithAttrs):
                         captures = ATTRS_MATCH(
                             string.encode('ascii', 'replace'), s, e).captures
                         row_attrs_append(dict(zip(
-                            captures('attr_name'), captures('attr_value')
-                        )))
+                            captures('attr_name'), captures('attr_value'))))
                 table_data = _apply_attr_spans(table_attrs, table_data)
         if row is None:
             if column is None:
@@ -224,7 +213,7 @@ class Table(SubWikiTextWithAttrs):
         for match_row in match_table:
             row_cells = []  # type: List[Cell]
             table_cells.append(row_cells)
-            header = match_row[0]['sep'] == '!'
+            header = match_row[0]['sep'] == b'!'
             if span:
                 row_attrs = []  # type: List[Dict[str, str]]
                 table_attrs.append(row_attrs)
@@ -241,24 +230,15 @@ class Table(SubWikiTextWithAttrs):
                     captures = attrs_match.captures
                     # noinspection PyUnboundLocalVariable
                     row_attrs_append(dict(zip(
-                        captures('attr_name'), captures('attr_value')
-                    )))
+                        captures('attr_name'), captures('attr_value'))))
                 old_span = next((s for s in spans if s == cell_span), None)
                 if old_span is None:
                     insort_right(spans, cell_span)
                 else:
                     cell_span = old_span
-                row_cells.append(
-                    Cell(
-                        self._lststr,
-                        header,
-                        type_to_spans,
-                        cell_span,
-                        type_,
-                        m,
-                        attrs_match,
-                    )
-                )
+                row_cells.append(Cell(
+                    self._lststr, header, type_to_spans, cell_span, type_, m,
+                    attrs_match))
         if table_cells and span:
             table_cells = _apply_attr_spans(table_attrs, table_cells)
         if row is None:
@@ -272,26 +252,24 @@ class Table(SubWikiTextWithAttrs):
     @property
     def caption(self) -> Optional[str]:
         """Caption of the table. Support get and set."""
-        m = CAPTION_MATCH(self.string)
+        m = CAPTION_MATCH(self._shadow)
         if m:
-            return m['caption']
+            return self(*m.span('caption'))
         return None
 
     @caption.setter
     def caption(self, newcaption: str) -> None:
-        m = CAPTION_MATCH(self.string)
+        shadow = self._shadow
+        m = CAPTION_MATCH(shadow)
         if m:
-            preattrs = m['preattrs']
-            attrs = m['attrs'] or ''
-            oldcaption = m['caption']
-            self[len(preattrs + attrs):len(preattrs + attrs + oldcaption)] =\
+            s = m.end('attrs')
+            self[s if s != -1 else m.end('preattrs'):m.end('caption')] =\
                 newcaption
-        else:
-            # There is no caption. Create one.
-            string = self.string
-            h, s, t = string.partition('\n')
-            # Insert caption after the first one.
-            self.insert(len(h + s), '|+' + newcaption + '\n')
+            return
+        # There is no caption. Create one.
+        h, s, t = shadow.partition(b'\n')
+        # Insert caption after the first one.
+        self.insert(len(h + s), '|+' + newcaption + '\n')
 
     @property
     def _attrs_match(self) -> Any:
@@ -307,24 +285,24 @@ class Table(SubWikiTextWithAttrs):
     @property
     def caption_attrs(self) -> Optional[str]:
         """Caption attributes. Support get and set operations."""
-        m = CAPTION_MATCH(self.string)
+        m = CAPTION_MATCH(self._shadow)
         if m:
-            return m['attrs']
+            s, e = m.span('attrs')
+            if s != -1:
+                return self(s, e)
         return None
 
     @caption_attrs.setter
     def caption_attrs(self, attrs: str) -> None:
-        string = self.string
-        h, s, t = string.partition('\n')
-        m = CAPTION_MATCH(string)
-        if not m:
-            # There is no caption-line
+        shadow = self._shadow
+        h, s, t = shadow.partition(b'\n')
+        m = CAPTION_MATCH(shadow)
+        if not m:  # There is no caption-line
             self.insert(len(h + s), '|+' + attrs + '|\n')
-        else:
-            preattrs = m['preattrs']
-            oldattrs = m['attrs'] or ''
-            # Caption and attrs or Caption but no attrs
-            self[len(preattrs):len(preattrs + oldattrs)] = attrs
+        else:  # Caption and attrs or Caption but no attrs
+            end = m.end('attrs')
+            if end != -1:
+                self[m.end('preattrs'):end] = attrs
 
 
 def _apply_attr_spans(
@@ -367,10 +345,7 @@ def _apply_attr_spans(
         for attrs, current_cell in zip(attrs_row, row):
             # 13.6
             attrs_get = attrs.get
-            while (
-                xcurrent < xwidth and
-                table[ycurrent][xcurrent] is not None
-            ):
+            while xcurrent < xwidth and table[ycurrent][xcurrent] is not None:
                 xcurrent += 1
             # 13.7
             if xcurrent == xwidth:
