@@ -6,34 +6,28 @@ from html import unescape
 from itertools import islice
 from operator import attrgetter
 from typing import (
-    Dict, Generator, Iterable, List, MutableSequence, Optional, Tuple, Type,
-    Union)
+    Dict, Generator, Iterable, List, MutableSequence, Optional, Tuple, Union)
 from warnings import warn
 
-from regex import VERBOSE, DOTALL, MULTILINE, IGNORECASE, search, finditer
-from regex import compile as regex_compile
+from regex import DOTALL, IGNORECASE, MULTILINE, VERBOSE, \
+    compile as regex_compile, finditer, search
 from wcwidth import wcswidth
 
 # noinspection PyProtectedMember
 from ._config import (
-    _tag_extensions, _HTML_TAG_NAME, _bare_external_link_schemes,
+    _HTML_TAG_NAME, _bare_external_link_schemes, _tag_extensions,
     regex_pattern)
 from ._spans import (
-    PARSABLE_TAG_EXTENSIONS_PATTERN,
-    START_TAG_PATTERN,
-    END_TAG_PATTERN,
-    parse_to_spans,
-    INVALID_URL_CHARS,
-    BARE_EXTERNAL_LINK,
-    EXTERNAL_LINK_URL_TAIL)
-
+    BARE_EXTERNAL_LINK, END_TAG_PATTERN, EXTERNAL_LINK_URL_TAIL,
+    INVALID_URL_CHARS, PARSABLE_TAG_EXTENSION_NAME, START_TAG_PATTERN,
+    parse_to_spans)
 
 NAME_CAPTURING_HTML_START_TAG_FINDITER = regex_compile(
     START_TAG_PATTERN.replace(
         b'{name}', rb'(?<name>' + _HTML_TAG_NAME + rb')', 1)).finditer
 
 PARSABLE_TAG_EXTENSIONS_MATCH = regex_compile(
-    rb'<' + PARSABLE_TAG_EXTENSIONS_PATTERN + rb'\b', IGNORECASE).match
+    rb'<' + PARSABLE_TAG_EXTENSION_NAME + rb'\b', IGNORECASE).match
 
 # External links
 BRACKET_EXTERNAL_LINK_SCHEMES = regex_pattern(
@@ -98,10 +92,6 @@ ITALIC_FINDITER = regex_compile(rb"""
     (?:'\0*+'|$)
 """, MULTILINE | VERBOSE).finditer
 
-BOLD_ITALIC_RECURSE_METHODS = (
-    'templates', 'parser_functions', 'parameters', '_extension_tags',
-    'wikilinks')
-
 # Types which are detected by parse_to_spans
 SPAN_PARSER_TYPES = {
     'Template', 'ParserFunction', 'WikiLink', 'Comment', 'Parameter',
@@ -544,12 +534,6 @@ class WikiText:
         for type_ in 'Comment', 'WikiLink':
             for s, e, _, _ in subspans(type_):
                 byte_array[s:e] = (e - s) * b'_'
-        for s, e, _, _ in subspans('ExtensionTag'):
-            if PARSABLE_TAG_EXTENSIONS_MATCH(byte_array, s, e):
-                byte_array[s:e] = b'  ' + INVALID_EXT_CHARS_SUB(
-                    b' ', byte_array[s + 2:e - 2]) + b'  '
-            else:
-                byte_array[s:e] = (e - s) * b'_'
         for type_ in 'Template', 'ParserFunction', 'Parameter':
             for s, e, _, _ in subspans(type_):
                 byte_array[s:e] = b'  ' + INVALID_EXT_CHARS_SUB(
@@ -581,10 +565,10 @@ class WikiText:
         replace_wikilinks=True,
         unescape_html_entities=True,
         replace_bolds_and_italics=True,
-        _mutate=False
+        _is_root_node=False
     ) -> str:
         """Return a plain text string representation of self."""
-        if _mutate is False:
+        if _is_root_node is False:
             s, e, m, b = self._span_data
             tts = self._inner_type_to_spans_copy()
             parsed = WikiText([self._lststr[0][s:e]], tts)
@@ -592,41 +576,68 @@ class WikiText:
         else:
             tts = self._type_to_spans
             parsed = self
-        for (b, e, _, _) in reversed(tts['Comment']):
-            del parsed[b:e]
+        lst = list(parsed.string)
+
+        def remove(b: int, e: int):
+            lst[b:e] = [None] * (e - b)
+
+        for (b, e, _, _) in tts['Comment']:
+            remove(b, e)
         if replace_templates:
-            for (b, e, _, _) in reversed(tts['Template']):
-                del parsed[b:e]
+            for (b, e, _, _) in tts['Template']:
+                remove(b, e)
         if replace_parser_functions:
-            for (b, e, _, _) in reversed(tts['ParserFunction']):
-                del parsed[b:e]
+            for (b, e, _, _) in tts['ParserFunction']:
+                remove(b, e)
         if replace_external_links:
-            for e in parsed.external_links:
-                if e.in_brackets:
-                    e[:] = e.text or ''
+            for el in parsed.external_links:
+                if el.in_brackets:
+                    b, e = el.span
+                    text = el.text
+                    if text is None:
+                        remove(b, e)
+                    else:
+                        remove(b, e - 1 - len(text))
+                        remove(e - 1, e)
         # replacing bold and italics should be done before wikilinks and tags
         # because removing tags and wikilinks creates invalid spans, and
         # get_bolds() will try to look into wikilinks for bold parts.
         if replace_bolds_and_italics:
-            for i in reversed(parsed.get_bolds_and_italics()):
-                i[:] = i.text
+            for i in parsed.get_bolds_and_italics():
+                b, e = i.span
+                ib, ie = i._match.span(1)  # text span
+                remove(b, b + ib)
+                remove(b + ie, e)
         if replace_parameters:
             for p in parsed.parameters:
-                default = p.default
-                if default is not None:
-                    p.string = default
+                b, e = p.span
+                default_start = p._shadow.find(124)
+                if default_start != -1:
+                    remove(b, b + default_start + 1)
+                    remove(e - 3, e)
                 else:
-                    del p.string
+                    remove(b, e)
         if replace_tags:
-            for t in reversed(parsed.get_tags()):
-                t[:] = t.contents
+            for t in parsed.get_tags():
+                b, e = t.span
+                cb, ce = t._match.span('contents')
+                remove(b, b + cb)
+                remove(b + ce, e)
         if replace_wikilinks:
-            for w in reversed(parsed.wikilinks):
+            for w in parsed.wikilinks:
+                b, e = w.span
                 if w.wikilinks:
-                    del w[:]  # image
+                    remove(b, e)  # image
                 else:
-                    w[:] = w.text or w.target
-        string = parsed.string
+                    tb, te = w._match.span(4)  # text span
+                    if tb != -1:
+                        remove(b, b + tb)
+                        remove(b + te, e)
+                    else:
+                        tb, te = w._match.span(1)  # target span
+                        remove(b, b + tb)
+                        remove(b + te, e)
+        string = ''.join(c for c in lst if c is not None)
         if unescape_html_entities:
             string = unescape(string)
         return string
@@ -917,6 +928,24 @@ class WikiText:
             odd_italics ^= True
         return shadow_copy
 
+    def _bolds_italics_recurse(self, result: list, filter_cls: Optional[type]):
+        for prop in (
+                'templates', 'parser_functions', 'parameters', 'wikilinks'):
+            for e in getattr(self, prop):
+                result += e.get_bolds_and_italics(
+                    filter_cls=filter_cls, recursive=False)
+        extension_tags = self._extension_tags
+        if not extension_tags:
+            return result
+        # noinspection PyProtectedMember
+        result_spans = {(*i._span_data[:2],) for i in result}
+        for e in extension_tags:
+            for i in e.get_bolds_and_italics(
+                    filter_cls=filter_cls, recursive=False):
+                # noinspection PyProtectedMember
+                if (*i._span_data[:2],) not in result_spans:
+                    result.append(i)
+
     def get_bolds_and_italics(
         self, *, recursive=True, filter_cls: type = None
     ) -> List[Union['Bold', 'Italic']]:
@@ -953,12 +982,12 @@ class WikiText:
                 else:
                     span = old_span
                 append(Bold(_lststr, type_to_spans, span, 'Bold'))
-            if not recursive:
-                return result
-            for m in BOLD_ITALIC_RECURSE_METHODS:
-                for e in getattr(self, m):
-                    result += e.get_bolds(False)
-            if filter_cls is Bold:
+            if recursive:
+                self._bolds_italics_recurse(result, filter_cls)
+                if filter_cls is Bold:
+                    result.sort(key=attrgetter('_span_data'))
+                    return result
+            elif filter_cls is Bold:
                 return result
         else:  # filter_cls is Italic
             bold_matches = BOLD_FINDITER(balanced_shadow, rs, re)
@@ -985,14 +1014,11 @@ class WikiText:
                 span = old_span
             append(Italic(
                 _lststr, type_to_spans, span, 'Bold', me != match.end(1)))
-        if not recursive:
+        if recursive and filter_cls is Italic:
+            self._bolds_italics_recurse(result, filter_cls)
             result.sort(key=attrgetter('_span_data'))
             return result
-        for m in BOLD_ITALIC_RECURSE_METHODS:
-            for e in getattr(self, m):
-                result += e.get_italics(False)
-
-        if filter_cls is None:  # the results are mixed and unsorted
+        if filter_cls is None:  # all Italics are appended after Bolds
             result.sort(key=attrgetter('_span_data'))
         return result
 
@@ -1039,16 +1065,24 @@ class WikiText:
         spans = type_to_spans.setdefault('ExternalLink', [])
         span_tuple_to_span_get = {(s[0], s[1]): s for s in spans}.get
         el_shadow = self._ext_link_shadow
-        for m in EXTERNAL_LINK_FINDITER(el_shadow):
-            ms, me = m.span()
-            span = s, e, _, _ = [ss + ms, ss + me, None, el_shadow[ms:me]]
-            old_span = span_tuple_to_span_get((s, e))
-            if old_span is None:
-                insort_right(spans, span)
-            else:
-                span = old_span
-            external_links_append(
-                ExternalLink(lststr, type_to_spans, span, 'ExternalLink'))
+
+        def _extract(start, end):
+            for m in EXTERNAL_LINK_FINDITER(el_shadow, start, end):
+                ms, me = m.span()
+                span = s, e, _, _ = [ss + ms, ss + me, None, el_shadow[ms:me]]
+                old_span = span_tuple_to_span_get((s, e))
+                if old_span is None:
+                    insort_right(spans, span)
+                else:
+                    span = old_span
+                external_links_append(
+                    ExternalLink(lststr, type_to_spans, span, 'ExternalLink'))
+
+        for s, e, _, _ in self._subspans('ExtensionTag'):
+            if PARSABLE_TAG_EXTENSIONS_MATCH(el_shadow, s, e):
+                _extract(s, e)
+            el_shadow[s:e] = (e - s) * b'_'
+        _extract(None, None)
         return external_links
 
     @property
@@ -1407,7 +1441,7 @@ def _outer_spans(sorted_spans: List[List[int]]) -> Iterable[List[int]]:
 
 def remove_markup(s: str, **kwargs) -> str:
     """Return a string with wiki markup removed/replaced."""
-    return WikiText(s).plain_text(**kwargs, _mutate=True)
+    return WikiText(s).plain_text(**kwargs, _is_root_node=True)
 
 
 plain_text_doc = """
